@@ -161,7 +161,7 @@ value make_pc loc erase_bef erase_aft is_empty_bef is_empty_aft pc bef bef_al
         else
           let bef = "%s" ^ bef in
           let e = <:expr< Pretty.sprintf $str:bef$ >> in
-          let e = <:expr< $e$ $pc$.bef >> in
+          let e = <:expr< $e$ $pc$.Pprintf.bef >> in
           List.fold_left (fun f e -> <:expr< $f$ $e$ >>) e bef_al
       in
       [(<:patt< Pprintf.bef >>, e)]
@@ -338,7 +338,15 @@ value rec expr_of_tree_aux loc fmt is_empty_bef is_empty_aft pc al t =
         expr_of_tree_aux loc fmt is_empty_bef False <:expr< pc >> al t1
       in
       let (e2, al) =
-        expr_of_tree_aux loc fmt True is_empty_aft pc al t2
+        let pc = if is_empty_bef then pc else <:expr< pc >> in
+        let (e2, al) =
+          expr_of_tree_aux loc fmt True is_empty_aft pc al t2
+        in
+        let e2 =
+          if is_empty_bef then e2
+          else <:expr< let pc = {($pc$) with Pprintf.bef = ""} in $e2$ >>
+        in
+        (e2, al)
       in
       let e = <:expr< let pc = {($pc$) with Pprintf.aft = $e2$} in $e1$ >> in
       (e, al)
