@@ -1,5 +1,5 @@
-(* camlp5r q_MLast.cmo ./pa_extfun.cmo ./pa_extprint.cmo *)
-(* $Id: pr_extend.ml 1804 2007-12-27 10:30:24Z deraugla $ *)
+(* camlp5r q_MLast.cmo ./pa_extfun.cmo ./pa_extprint.cmo ./pa_pprintf.cmo *)
+(* $Id: pr_extend.ml 1856 2008-01-04 13:53:50Z deraugla $ *)
 (* Copyright (c) INRIA 2007-2008 *)
 
 (* heuristic to rebuild the EXTEND statement from the AST *)
@@ -14,17 +14,16 @@ Pcaml.strict_mode.val := True;
 value not_impl name pc x =
   let desc =
     if Obj.tag (Obj.repr x) = Obj.tag (Obj.repr "") then
-      sprintf "\"%s\"" (Obj.magic x)
+      "\"" ^ Obj.magic x ^ "\""
     else if Obj.is_block (Obj.repr x) then
       "tag = " ^ string_of_int (Obj.tag (Obj.repr x))
     else "int_val = " ^ string_of_int (Obj.magic x)
   in
-  sprintf "%s\"pr_extend, not impl: %s; %s\"%s" pc.bef name
-    (String.escaped desc) pc.aft
+  pprintf pc "\"pr_extend, not impl: %s; %s\"" name (String.escaped desc)
 ;
 
-value bar_before elem pc x = elem {(pc) with bef = sprintf "%s| " pc.bef} x;
-value semi_after elem pc x = elem {(pc) with aft = sprintf ";%s" pc.aft} x;
+value bar_before elem pc x = pprintf pc "| %p" elem x;
+value semi_after elem pc x = pprintf pc "%p;" elem x;
 
 (* Extracting *)
 
@@ -72,6 +71,7 @@ value unposition =
   | <:expr< Some Gramext.Last >> -> Some Gramext.Last
   | <:expr< Some (Gramext.Before $str:s$) >> -> Some (Gramext.Before s)
   | <:expr< Some (Gramext.After $str:s$) >> -> Some (Gramext.After s)
+  | <:expr< Some (Gramext.Like $str:s$) >> -> Some (Gramext.Like s)
   | <:expr< Some (Gramext.Level $str:s$) >> -> Some (Gramext.Level s)
   | _ -> raise Not_found ]
 ;
@@ -245,23 +245,17 @@ value flag_equilibrate_cases = Pcaml.flag_equilibrate_cases;
 value expr = Eprinter.apply pr_expr;
 value patt = Eprinter.apply pr_patt;
 
-value string pc s = sprintf "%s\"%s\"%s" pc.bef s pc.aft;
+value string pc s = pprintf pc "\"%s\"" s;
 
 value position pc pos =
   match pos with
-  [ None -> sprintf "%s%s" pc.bef pc.aft
-  | Some Gramext.First -> sprintf "%s FIRST%s" pc.bef pc.aft
-  | Some Gramext.Last -> sprintf "%s LAST%s" pc.bef pc.aft
-  | Some (Gramext.Before s) -> sprintf "%s BEFORE%s" pc.bef pc.aft
-  | Some (Gramext.After s) ->
-      sprintf "%s AFTER %s%s" pc.bef (string {(pc) with bef = ""; aft = ""} s)
-        pc.aft
-  | Some (Gramext.Like s) ->
-      sprintf "%s LIKE %s%s" pc.bef (string {(pc) with bef = ""; aft = ""} s)
-        pc.aft
-  | Some (Gramext.Level s) ->
-      sprintf "%s LEVEL %s%s" pc.bef (string {(pc) with bef = ""; aft = ""} s)
-        pc.aft ]
+  [ None -> pprintf pc ""
+  | Some Gramext.First -> pprintf pc " FIRST"
+  | Some Gramext.Last -> pprintf pc " LAST"
+  | Some (Gramext.Before s) -> pprintf pc " BEFORE"
+  | Some (Gramext.After s) -> pprintf pc " AFTER %p" string s
+  | Some (Gramext.Like s) -> pprintf pc " LIKE %p" string s
+  | Some (Gramext.Level s) -> pprintf pc " LEVEL %p" string s ]
 ;
 
 value action expr pc a = expr pc a;
@@ -270,23 +264,17 @@ value token pc tok =
   match tok with
   [ Left (con, prm) ->
       if con = "" then string pc prm
-      else if prm = "" then sprintf "%s%s%s" pc.bef con pc.aft
-      else
-        sprintf "%s%s %s%s" pc.bef con
-          (string {(pc) with bef = ""; aft = ""} prm) pc.aft
-  | Right <:expr< ("", $x$) >> ->
-      sprintf "%s$%s$%s" pc.bef (expr {(pc) with bef = ""; aft = ""} x)
-        pc.aft
-  | Right <:expr< ($str:con$, $x$) >> ->
-      sprintf "%s%s $%s$%s" pc.bef con (expr {(pc) with bef = ""; aft = ""} x)
-        pc.aft
+      else if prm = "" then pprintf pc "%s" con
+      else pprintf pc "%s %p" con string prm
+  | Right <:expr< ("", $x$) >> -> pprintf pc "$%p$" expr x
+  | Right <:expr< ($str:con$, $x$) >> -> pprintf pc "%s $%p$" con expr x
   | Right _ -> assert False ]
 ;
 
-value rec string_list =
+value rec string_list pc =
   fun
-  [ [s :: sl] -> sprintf " \"%s\"%s" s (string_list sl)
-  | [] -> "" ]
+  [ [s :: sl] -> pprintf pc " \"%s\"%p" s string_list sl
+  | [] -> pprintf pc "" ]
 ;
 
 value anti_anti n =
@@ -313,160 +301,104 @@ value anti_of_tok =
   | s -> [] ]
 ;
 
+value comment pc loc = pprintf pc "%s" (Prtools.comm_bef pc.ind loc);
+
 value rec rule force_vertic pc (sl, a) =
   match a with
   [ None -> not_impl "rule 1" pc sl
   | Some a ->
       if sl = [] then
-        action expr
-          {(pc) with ind = pc.ind + 4;
-           bef =
-             sprintf "%s->%s " pc.bef (comm_bef pc (MLast.loc_of_expr a));
-           dang = "|"}
-          a
+        pprintf pc "@[<4>->%p %q@]" comment (MLast.loc_of_expr a)
+          (action expr) a "|"
       else
         match
           horiz_vertic
             (fun () ->
                let s =
-                 hlistl (semi_after psymbol) psymbol
-                   {(pc) with bef = ""; aft = ""} sl
+                 let pc = {(pc) with aft = ""} in
+                 pprintf pc "%p ->" (hlistl (semi_after psymbol) psymbol) sl
                in
-               Some (sprintf "%s%s ->" pc.bef s))
+               Some s)
             (fun () -> None)
         with
         [ Some s1 ->
+            let pc = {(pc) with bef = ""} in
             horiz_vertic
               (fun () ->
                  if force_vertic then sprintf "\n"
-                 else
-                   sprintf "%s %s%s" s1
-                     (action expr {(pc) with bef = ""; aft = ""; dang = "|"}
-                        a)
-                     pc.aft)
+                 else pprintf pc "%s %q" s1 (action expr) a "|")
               (fun () ->
-                 let s2 =
-                   action expr
-                     {(pc) with ind = pc.ind + 4; bef = tab (pc.ind + 4);
-                      dang = "|"}
-                     a
-                 in
-                 sprintf "%s\n%s" s1 s2)
+                 pprintf pc "%s@;<1 4>%q" s1 (action expr) a "|")
         | None ->
             let sl = List.map (fun s -> (s, ";")) sl in
-            let s1 =
-              plist psymbol 0 {(pc) with ind = pc.ind + 2; aft = " ->"} sl
-            in
-            let s2 =
-              action expr
-                {(pc) with ind = pc.ind + 4; bef = tab (pc.ind + 4);
-                 dang = "|"}
-                a
-            in
-            sprintf "%s\n%s" s1 s2 ] ]
+            pprintf pc "@[<2>%p ->@;%q@]" (plist psymbol 0) sl
+              (action expr) a "|" ] ]
 and psymbol pc (p, s) =
   match p with
   [ None -> symbol pc s
-  | Some p ->
-      horiz_vertic
-        (fun () ->
-           sprintf "%s%s = %s%s" pc.bef
-             (pattern {(pc) with bef = ""; aft = ""} p)
-             (symbol {(pc) with bef = ""; aft = ""} s) pc.aft)
-        (fun () ->
-           let s1 = pattern {(pc) with aft = " ="} p in
-           let s2 =
-             symbol {(pc) with ind = pc.ind + 2; bef = tab (pc.ind + 2)} s
-           in
-           sprintf "%s\n%s" s1 s2) ]
+  | Some p -> pprintf pc "%p =@;%p" pattern p symbol s ]
 and pattern pc p =
   match p with
-  [ <:patt< $lid:i$ >> -> sprintf "%s%s%s" pc.bef i pc.aft
-  | <:patt< _ >> -> sprintf "%s_%s" pc.bef pc.aft
+  [ <:patt< $lid:i$ >> -> pprintf pc "%s" i
+  | <:patt< _ >> -> pprintf pc "_"
   | <:patt< ($list:pl$) >> ->
       let pl = List.map (fun p -> (p, ",")) pl in
-      plist patt 1
-        {(pc) with bef = sprintf "%s(" pc.bef; aft = sprintf ")%s" pc.aft}
-        pl
+      pprintf pc "(%p)" (plist patt 1) pl
   | p ->
-      patt
-        {(pc) with ind = pc.ind + 1; bef = sprintf "%s(" pc.bef;
-         aft = sprintf ")%s" pc.aft}
-        p ]
+      pprintf pc "@[<1>(%p)@]" patt p ]
 and symbol pc sy =
   match sy with
   [ Snterm e -> expr pc e
-  | Snterml e s -> expr {(pc) with aft = sprintf " LEVEL \"%s\"%s" s pc.aft} e
+  | Snterml e s -> pprintf pc "%p LEVEL \"%s\"" expr e s
   | Slist0 sy ->
-      sprintf "%sLIST0 %s" pc.bef (simple_symbol {(pc) with bef = ""} sy)
+      pprintf pc "LIST0@;%p" simple_symbol sy
   | Slist0sep sy sep ->
-      sprintf "%sLIST0 %s SEP %s" pc.bef
-        (simple_symbol {(pc) with bef = ""; aft = ""} sy)
-        (simple_symbol {(pc) with bef = ""} sep)
+      pprintf pc "LIST0@;%p@ @[SEP@;%p@]" simple_symbol sy simple_symbol sep
   | Slist1 sy ->
-      sprintf "%sLIST1 %s" pc.bef (simple_symbol {(pc) with bef = ""} sy)
+      pprintf pc "LIST1@;%p" simple_symbol sy
   | Slist1sep sy sep ->
-      sprintf "%sLIST1 %s SEP %s" pc.bef
-        (simple_symbol {(pc) with bef = ""; aft = ""} sy)
-        (simple_symbol {(pc) with bef = ""} sep)
+      pprintf pc "LIST1@;%p@ @[SEP@;%p@]" simple_symbol sy simple_symbol sep
   | Sopt sy ->
-      sprintf "%sOPT %s" pc.bef (simple_symbol {(pc) with bef = ""} sy)
+      pprintf pc "OPT@;%p" simple_symbol sy
   | Sflag sy ->
-      sprintf "%sFLAG %s" pc.bef (simple_symbol {(pc) with bef = ""} sy)
+      pprintf pc "FLAG@;%p" simple_symbol sy
   | Srules rl ->
       match check_slist rl with
       [ Some s -> s_symbol pc s
       | None -> simple_symbol pc sy ]
   | Stoken tok -> token pc tok
   | Svala sl _ sy ->
-      sprintf "%sV %s%s%s" pc.bef
-        (simple_symbol {(pc) with bef = ""; aft = ""} sy) (string_list sl)
-        pc.aft
+      pprintf pc "V @[<2>%p%p@]" simple_symbol sy string_list sl
   | sy -> simple_symbol pc sy ]
 and simple_symbol pc sy =
   match sy with
-  [ Snterm <:expr< $lid:s$ >> -> sprintf "%s%s%s" pc.bef s pc.aft
-  | Sself -> sprintf "%sSELF%s" pc.bef pc.aft
-  | Snext -> sprintf "%sNEXT%s" pc.bef pc.aft
+  [ Snterm <:expr< $lid:s$ >> -> pprintf pc "%s" s
+  | Sself -> pprintf pc "SELF"
+  | Snext -> pprintf pc "NEXT"
   | Srules rl ->
       match check_slist rl with
-      [ Some _ ->
-          symbol
-            {(pc) with bef = sprintf "%s(" pc.bef; aft = sprintf ")%s" pc.aft}
-            sy
+      [ Some _ -> pprintf pc "(%p)" symbol sy
       | None ->
           horiz_vertic
             (fun () ->
-               hlist2 (rule False) (bar_before (rule False))
-                 {(pc) with bef = sprintf "%s[ " pc.bef;
-                  aft = sprintf " ]%s" pc.aft}
-                 rl)
+               pprintf pc "[ %p ]"
+                 (hlist2 (rule False) (bar_before (rule False))) rl)
             (fun () ->
-               vlist2 (rule False) (bar_before (rule False))
-                 {(pc) with bef = sprintf "%s[ " pc.bef;
-                  aft = sprintf " ]%s" pc.aft}
-                 rl) ]
+               pprintf pc "[ %p ]"
+                 (vlist2 (rule False) (bar_before (rule False))) rl) ]
   | Stoken (Left ("", _) | Left (_, "")) -> symbol pc sy
   | Snterml _ _ | Slist0 _ | Slist0sep _ _ | Slist1 _ | Slist1sep _ _ |
     Sflag _ | Sopt _ ->
-      symbol
-        {(pc) with bef = sprintf "%s(" pc.bef; aft = sprintf ")%s" pc.aft}
-        sy
+      pprintf pc "@[<1>(%p)@]" symbol sy
   | sy -> not_impl "simple_symbol" pc sy ]
 and s_symbol pc =
   fun
-  [ Slist0 sy ->
-      sprintf "%sSLIST0 %s" pc.bef (simple_symbol {(pc) with bef = ""} sy)
-  | Slist1 sy ->
-      sprintf "%sSLIST1 %s" pc.bef (simple_symbol {(pc) with bef = ""} sy)
+  [ Slist0 sy -> pprintf pc "SLIST0@;%p" simple_symbol sy
+  | Slist1 sy -> pprintf pc "SLIST1@;%p" simple_symbol sy
   | Slist0sep sy sep ->
-      sprintf "%sSLIST0 %s SEP %s" pc.bef
-        (simple_symbol {(pc) with bef = ""; aft = ""} sy)
-        (simple_symbol {(pc) with bef = ""} sep)
+      pprintf pc "SLIST0@;%p@ @[SEP@;%p@]" simple_symbol sy simple_symbol sep
   | Slist1sep sy sep ->
-      sprintf "%sSLIST1 %s SEP %s" pc.bef
-        (simple_symbol {(pc) with bef = ""; aft = ""} sy)
-        (simple_symbol {(pc) with bef = ""} sep)
+      pprintf pc "SLIST1@;%p@ @[SEP@;%p@]" simple_symbol sy simple_symbol sep
   | Sopt s ->
       let sy =
         match s with
@@ -476,16 +408,14 @@ and s_symbol pc =
             Stoken (Left ("", str))
         | s -> s ]
       in
-      sprintf "%sSOPT %s" pc.bef (simple_symbol {(pc) with bef = ""} sy)
+      pprintf pc "SOPT@;%p" simple_symbol sy
   | Sflag sy ->
-      sprintf "%sSFLAG %s" pc.bef (simple_symbol {(pc) with bef = ""} sy)
+      pprintf pc "SFLAG@;%p" simple_symbol sy
   | Svala ls oe s ->
-      sprintf "%sSV %s%s%s%s" pc.bef
-        (simple_symbol {(pc) with bef = ""; aft= ""} s) (string_list ls)
+      pprintf pc "SV@;%p%p%s" simple_symbol s string_list ls
         (match oe with
          [ Some e -> " " ^ e
          | None -> "" ])
-        pc.aft
   | _ -> assert False ]
 and check_slist rl =
   if no_slist.val then None
@@ -547,46 +477,37 @@ and check_slist rl =
           | _ -> None ] ]
 ;
 
-value label =
+value label pc =
   fun
-  [ Some s -> sprintf "\"%s\"" s
-  | None -> "" ]
+  [ Some s -> pprintf pc "\"%s\"" s
+  | None -> pprintf pc "" ]
 ;
 
-value assoc =
+value assoc pc =
   fun
-  [ Some Gramext.NonA -> "NONA"
-  | Some Gramext.LeftA -> "LEFTA"
-  | Some Gramext.RightA -> "RIGHTA"
-  | None -> "" ]
+  [ Some Gramext.NonA -> pprintf pc "NONA"
+  | Some Gramext.LeftA -> pprintf pc "LEFTA"
+  | Some Gramext.RightA -> pprintf pc "RIGHTA"
+  | None -> pprintf pc "" ]
 ;
 
 value level force_vertic pc (lab, ass, rl) =
   match (lab, ass) with
   [ (None, None) ->
-      if rl = [] then sprintf "%s[ ]%s" pc.bef pc.aft
+      if rl = [] then pprintf pc "[ ]"
       else
-        vlist2 (rule force_vertic) (bar_before (rule force_vertic))
-          {(pc) with ind = pc.ind + 2; bef = sprintf "%s[ " pc.bef;
-           aft = sprintf " ]%s" pc.aft}
-          rl
+        pprintf pc "@[<2>[ %p ]@]"
+          (vlist2 (rule force_vertic) (bar_before (rule force_vertic))) rl
   | _ ->
-      let s1 =
-        match (lab, ass) with
-        [ (Some _, None) -> sprintf "%s%s" pc.bef (label lab)
-        | (None, Some _) -> sprintf "%s%s" pc.bef (assoc ass)
-        | (Some _, Some _) -> sprintf "%s%s %s" pc.bef (label lab) (assoc ass)
-        | _ -> assert False ]
-      in
-      let s2 =
-        if rl = [] then not_impl "level" {(pc) with bef = ""} rl
-        else
-          vlist2 (rule force_vertic) (bar_before (rule force_vertic))
-            {(pc) with ind = pc.ind + 2;
-             bef = sprintf "%s[ " (tab (pc.ind + 2));
-             aft = sprintf " ]%s" pc.aft} rl
-      in
-      sprintf "%s\n%s" s1 s2 ]
+      pprintf pc "@[<b>%p@;[ %p ]@]"
+        (fun pc ->
+           fun
+           [ (Some _, None) -> label pc lab
+           | (None, Some _) -> assoc pc ass
+            | (Some _, Some _) -> pprintf pc "%p %p" label lab assoc ass
+            | _ -> assert False ])
+        (lab, ass)
+        (vlist2 (rule force_vertic) (bar_before (rule force_vertic))) rl ]
 ;
 
 value entry pc (e, pos, ll) =
@@ -609,14 +530,9 @@ value entry pc (e, pos, ll) =
       has_vertic
     else False
   in
-  sprintf "%s%s%s:%s\n%s\n%s;%s" (comm_bef pc (MLast.loc_of_expr e)) pc.bef
-    (expr {(pc) with bef = ""; aft = ""} e)
-    (position {(pc) with bef = ""; aft = ""} pos)
-    (vlist2 (level force_vertic) (bar_before (level force_vertic))
-       {(pc) with ind = pc.ind + 2; bef = sprintf "%s[ " (tab (pc.ind + 2));
-        aft = " ]"}
-        ll)
-    (tab pc.ind) pc.aft
+  comm_bef pc.ind (MLast.loc_of_expr e) ^
+  pprintf pc "@[<b>%p:%p@;[ %p ]@ ;@]" expr e position pos
+    (vlist2 (level force_vertic) (bar_before (level force_vertic))) ll
 ;
 
 value extend_body pc (globals, entries) =
@@ -624,12 +540,8 @@ value extend_body pc (globals, entries) =
   [ [] -> vlist entry pc entries
   | _ ->
       let globals = List.map (fun g -> (g, "")) globals in
-      let s1 =
-        plist expr 2 {(pc) with bef = sprintf "%sGLOBAL: " pc.bef; aft = ";"}
-          globals
-      in
-      let s2 = vlist entry {(pc) with bef = tab pc.ind} entries in
-      sprintf "%s\n%s" s1 s2 ]
+      pprintf pc "@[<b>GLOBAL: %p;@ %p@]" (plist expr 2) globals
+        (vlist entry) entries ]
 ;
 
 value extend pc e =
@@ -637,19 +549,9 @@ value extend pc e =
   [ <:expr< Grammar.extend $e$ >> ->
       try
         let ex = unextend_body e in
-        let s =
-          extend_body
-            {(pc) with ind = pc.ind + 2; bef = tab (pc.ind + 2); aft = ""} ex
-        in
-        sprintf "%sEXTEND\n%s\n%sEND%s" pc.bef s (tab pc.ind) pc.aft
+        pprintf pc "EXTEND@;%p@ END" extend_body ex
       with
-      [ Not_found ->
-          sprintf "%sGrammar.extend\n%s" pc.bef
-            (expr
-               {(pc) with ind = pc.ind + 2;
-                bef = sprintf "%s(" (tab (pc.ind + 2));
-                aft = sprintf ")%s" pc.aft}
-               e) ]
+      [ Not_found -> pprintf pc "Grammar.extend@;@[<1>(%p)@]" expr e ]
   | e -> expr pc e ]
 ;
 
