@@ -1,12 +1,12 @@
 (* camlp5r pa_macro.cmo q_MLast.cmo ./pa_extfun.cmo ./pa_extprint.cmo ./pa_pprintf.cmo *)
-(* $Id: pr_o.ml 1822 2007-12-28 12:58:30Z deraugla $ *)
+(* $Id: pr_o.ml 1896 2008-01-23 12:46:58Z deraugla $ *)
 (* Copyright (c) INRIA 2007-2008 *)
 
 open Pretty;
 open Pcaml;
 open Prtools;
 
-value flag_comments_in_phrases = ref True;
+value flag_comments_in_phrases = Pcaml.flag_comments_in_phrases;
 value flag_equilibrate_cases = Pcaml.flag_equilibrate_cases;
 value flag_horiz_let_in = ref True;
 value flag_semi_semi = ref False;
@@ -100,11 +100,6 @@ value rec get_defined_ident =
   | _ -> [] ]
 ;
 
-(**)
-value test = ref False;
-Pcaml.add_option "-test" (Arg.Set test) " test";
-(**)
-
 value not_impl name pc x =
   let desc =
     if Obj.tag (Obj.repr x) = Obj.tag (Obj.repr "") then
@@ -208,13 +203,13 @@ value comm_bef pc loc =
 
 (* expression with adding the possible comment before *)
 value comm_expr expr pc z =
-  let ccc = comm_bef pc (MLast.loc_of_expr z) in
+  let ccc = comm_bef pc.ind (MLast.loc_of_expr z) in
   sprintf "%s%s" ccc (expr pc z)
 ;
 
 (* couple pattern/anytype with adding the possible comment before *)
 value comm_patt_any f pc z =
-  let ccc = comm_bef pc (MLast.loc_of_patt (fst z)) in
+  let ccc = comm_bef pc.ind (MLast.loc_of_patt (fst z)) in
   sprintf "%s%s" ccc (f pc z)
 ;
 
@@ -464,14 +459,6 @@ value has_cons_with_params vdl =
     vdl
 ;
 
-value rec get_else_if =
-  fun
-  [ <:expr< if $e1$ then $e2$ else $e3$ >> ->
-      let (eel, e3) = get_else_if e3 in
-      ([(e1, e2) :: eel], e3)
-  | e -> ([], e) ]
-;
-
 value alone_in_line pc =
   (pc.aft = "" || pc.aft = ";") && pc.bef <> "" &&
   loop 0 where rec loop i =
@@ -514,6 +501,86 @@ value are_close f x1 x2 =
   | _ -> False ]
 ;
 
+(* if statement *)
+
+value rec get_else_if =
+  fun
+  [ <:expr< if $e1$ then $e2$ else $e3$ >> ->
+      let (eel, e3) = get_else_if e3 in
+      ([(e1, e2) :: eel], e3)
+  | e -> ([], e) ]
+;
+
+value if_then force_vertic curr pc (e1, e2) =
+  horiz_vertic
+    (fun () ->
+       if force_vertic then sprintf "\n"
+       else pprintf pc "if %q then %p" curr e1 "" curr e2)
+    (fun () ->
+       pprintf pc "@[<3>if %q@]@ then@;%p" curr e1 "" (comm_expr expr1) e2)
+;
+
+value else_if_then force_vertic curr pc (e1, e2) =
+  horiz_vertic
+    (fun () ->
+       if force_vertic then sprintf "\n"
+       else
+         pprintf pc "else if %q then %p" curr e1 ""
+           curr e2)
+    (fun () ->
+       pprintf pc "@[<a>else if@;%q@ then@]@;%p" curr e1 ""
+         (comm_expr expr1) e2)
+;
+
+value loop_else_if_no_else force_vertic curr pc eel =
+  loop pc eel where rec loop pc =
+    fun
+    [ [(e1, e2)] ->
+        pprintf pc "@[<b>@ %p@]" (else_if_then force_vertic curr) (e1, e2)
+    | [(e1, e2) :: eel] ->
+        pprintf pc "@[<b>@ %q%p@]"
+          (else_if_then force_vertic curr) (e1, e2) "else" loop eel
+    | [] ->
+        pprintf pc "" ]
+;
+
+value loop_else_if_and_else force_vertic curr pc (eel, e3) =
+  loop pc eel where rec loop pc =
+    fun
+    [ [(e1, e2) :: eel] ->
+        pprintf pc "@[<b>@ %q%p@]"
+          (else_if_then force_vertic curr) (e1, e2) "else" loop eel
+    | [] ->
+        pprintf pc "@[<b>@ @[else@;%p@]@]" (comm_expr curr) e3 ]
+;
+
+value if_case_has_vertic curr pc e1 e2 eel e3 =
+  horiz_vertic
+    (fun () ->
+       let _ : string = if_then False curr {(pc) with aft = ""} (e1, e2) in
+       False)
+    (fun () -> True) ||
+  List.exists
+    (fun (e1, e2) ->
+       horiz_vertic
+         (fun () ->
+            let _ : string =
+              else_if_then False curr {(pc) with bef = tab pc.ind; aft = ""}
+                (e1, e2)
+            in
+            False)
+         (fun () -> True))
+    eel ||
+  horiz_vertic
+    (fun () ->
+       let _ : string =
+         let pc = {(pc) with bef = tab pc.ind} in
+         pprintf pc "else %p" (comm_expr curr) e3
+       in
+       False)
+    (fun () -> True)
+;
+
 (* Expressions displayed without spaces separating elements; special
    for expressions as strings or arrays indexes (x.[...] or x.(...)).
    Applied only if only containing +, -, *, /, integers and variables. *)
@@ -697,21 +764,6 @@ EXTEND_PRINTER
                    pprintf pc "if %q then %q else %p" curr e1 "" curr e2 ""
                      curr e3 ])
             (fun () ->
-               let if_then force_vertic pc else_b e1 e2 =
-                 horiz_vertic
-                   (fun () ->
-                      if force_vertic then sprintf "\n"
-                      else
-                        pprintf pc "%sif %q then %p" else_b curr e1 ""
-                          curr e2)
-                   (fun () ->
-                      if else_b = "" then
-                        pprintf pc "@[<3>%sif %q@]@ then@;%p" else_b
-                          curr e1 "" (comm_expr expr1) e2
-                      else
-                        pprintf pc "@[<a>%sif@;%q@ then@]@;%p" else_b
-                          curr e1 "" (comm_expr expr1) e2)
-               in
                let (force_vertic, eel, e3) =
                  if flag_equilibrate_cases.val then
                    let (eel, e3) =
@@ -725,80 +777,24 @@ EXTEND_PRINTER
                      else get_else_if e3
                    in
                    (* if a case does not fit on line, all cases must be cut *)
-                   let has_vertic =
-                     horiz_vertic
-                       (fun () ->
-                          let _ : string =
-                            if_then False {(pc) with aft = ""} "" e1 e2
-                          in
-                          False)
-                       (fun () -> True) ||
-                     List.exists
-                       (fun (e1, e2) ->
-                          horiz_vertic
-                            (fun () ->
-                               let _ : string =
-                                 if_then False
-                                   {(pc) with bef = tab pc.ind; aft = ""}
-                                   "else " e1 e2
-                               in
-                               False)
-                            (fun () -> True))
-                       eel ||
-                     horiz_vertic
-                       (fun () ->
-                          let _ : string =
-                            let pc = {(pc) with bef = tab pc.ind} in
-                            pprintf pc "else %p" (comm_expr curr) e3
-                          in
-                          False)
-                       (fun () -> True)
-                   in
+                   let has_vertic = if_case_has_vertic curr pc e1 e2 eel e3 in
                    (has_vertic, eel, e3)
                  else
                    let (eel, e3) = get_else_if e3 in
                    (False, eel, e3)
                in
                match e3 with
-               [ <:expr< () >> when pc.dang = "else" -> next pc ge
+               [ <:expr< () >> ->
+                   if pc.dang = "else" then next pc ge
+                   else if eel = [] then if_then force_vertic curr pc (e1, e2)
+                   else
+                     pprintf pc "%q%p"
+                       (if_then force_vertic curr) (e1, e2) "else"
+                       (loop_else_if_no_else force_vertic curr) eel
                | _ ->
-                   let s1 =
-                     let (pc_dang, pc_aft) =
-                       match (eel, e3) with
-                       [ ([], <:expr< () >>) -> (pc.dang, pc.aft)
-                       | _ -> ("else", "") ]
-                     in
-                     if_then force_vertic
-                       {(pc) with aft = pc_aft; dang = pc_dang} "" e1 e2
-                   in
-                   let s2 =
-                     loop eel where rec loop =
-                       fun
-                       [ [(e1, e2) :: eel] ->
-                           let (pc_dang, pc_aft) =
-                             match (eel, e3) with
-                             [ ([], <:expr< () >>) -> (pc.dang, pc.aft)
-                             | _ -> ("else", "") ]
-                           in
-                           sprintf "\n%s%s"
-                             (if_then force_vertic
-                                {(pc) with bef = tab pc.ind; aft = pc_aft;
-                                 dang = pc_dang}
-                                "else " e1 e2)
-                             (loop eel)
-                       | [] -> "" ]
-                   in
-                   let s3 =
-                     match e3 with
-                     [ <:expr< () >> -> ""
-                     | _ ->
-                         let s =
-                           let pc = {(pc) with bef = tab pc.ind} in
-                           pprintf pc "else@;%p" (comm_expr curr) e3
-                         in
-                         sprintf "\n%s" s ]
-                   in
-                   sprintf "%s%s%s" s1 s2 s3 ])
+                   pprintf pc "%q%p"
+                     (if_then force_vertic curr) (e1, e2) "else"
+                     (loop_else_if_and_else force_vertic curr) (eel, e3) ])
       | <:expr< fun [ $list:pwel$ ] >> as ge ->
           match pwel with
           [ [(p1, <:vala< None >>, e1)] when is_irrefut_patt p1 ->
@@ -1101,6 +1097,7 @@ EXTEND_PRINTER
         <:expr< for $lid:_$ = $_$ $to:_$ $_$ do { $list:_$ } >> |
         <:expr< while $_$ do { $list:_$ } >> |
         <:expr< let $flag:_$ $list:_$ in $_$ >> |
+        <:expr< let module $uid:_$ = $_$ in $_$ >> |
         <:expr< match $_$ with [ $list:_$ ] >> |
         <:expr< try $_$ with [ $list:_$ ] >> as z ->
           pprintf pc "@[<1>(%q)@]" expr z "" ] ]
@@ -1230,13 +1227,15 @@ EXTEND_PRINTER
           pprintf pc "@[<a>@[<2>{ %p }@]@]"
             (hvlistl (semi_after label_decl) label_decl) ltl
       | <:ctyp< [ $list:vdl$ ] >> ->
-          horiz_vertic
-            (fun () ->
-               if has_cons_with_params vdl then sprintf "\n"
-               else hlist2 cons_decl (bar_before cons_decl) pc vdl)
-            (fun () ->
-               pprintf pc "  %p" (vlist2 cons_decl (bar_before cons_decl))
-                 vdl)
+          if vdl = [] then pprintf pc "[]"
+          else
+            horiz_vertic
+              (fun () ->
+                 if has_cons_with_params vdl then sprintf "\n"
+                 else hlist2 cons_decl (bar_before cons_decl) pc vdl)
+              (fun () ->
+                 pprintf pc "  %p" (vlist2 cons_decl (bar_before cons_decl))
+                   vdl)
       | <:ctyp< $lid:t$ >> ->
           var_escaped pc t
       | <:ctyp< $uid:t$ >> ->
@@ -1375,7 +1374,9 @@ EXTEND_PRINTER
           pprintf pc "%s" s
       | <:module_expr< ($me$ : $mt$) >> ->
           pprintf pc "@[<1>(%p :@ %p)@]" module_expr me module_type mt
-      | <:module_expr< struct $list:_$ end >> as z ->
+      | <:module_expr< functor ($uid:_$ : $_$) -> $_$ >> |
+        <:module_expr< struct $list:_$ end >> | <:module_expr< $_$ . $_$ >> |
+        <:module_expr< $_$ $_$ >> as z ->
           pprintf pc "@[<1>(%p)@]" module_expr z ] ]
   ;
   pr_module_type:
