@@ -1,12 +1,12 @@
-(* camlp5r pa_macro.cmo q_MLast.cmo ./pa_extfun.cmo ./pa_extprint.cmo ./pa_pprintf.cmo *)
-(* $Id: pr_r.ml 1843 2007-12-30 10:44:34Z deraugla $ *)
+(* camlp5r -I . pa_macro.cmo q_MLast.cmo pa_extfun.cmo pa_extprint.cmo pa_pprintf.cmo *)
+(* $Id: pr_r.ml 1896 2008-01-23 12:46:58Z deraugla $ *)
 (* Copyright (c) INRIA 2007-2008 *)
 
 open Pretty;
 open Pcaml;
 open Prtools;
 
-value flag_comments_in_phrases = ref True;
+value flag_comments_in_phrases = Pcaml.flag_comments_in_phrases;
 value flag_expand_declare = ref False;
 value flag_horiz_let_in = ref False;
 value flag_sequ_begin_at_eol = ref True;
@@ -224,13 +224,13 @@ value comm_bef pc loc =
 
 (* expression with adding the possible comment before *)
 value comm_expr expr pc z =
-  let ccc = comm_bef pc (MLast.loc_of_expr z) in
+  let ccc = comm_bef pc.ind (MLast.loc_of_expr z) in
   sprintf "%s%s" ccc (expr pc z)
 ;
 
 (* couple pattern/anytype with adding the possible comment before *)
 value comm_patt_any f pc z =
-  let ccc = comm_bef pc (MLast.loc_of_patt (fst z)) in
+  let ccc = comm_bef pc.ind (MLast.loc_of_patt (fst z)) in
   sprintf "%s%s" ccc (f pc z)
 ;
 
@@ -302,16 +302,9 @@ value rec where_binding pc (p, e, body) =
   let pl = [p :: pl] in
   match sequencify body with
   [ Some el ->
-      horiz_vertic
-        (fun () ->
-           pprintf pc "%p@ where rec %p =@;%p" expr e (hlist patt) pl
-             (comm_expr expr) body)
-        (fun () ->
-           let expr_wh =
-             if flag_where_in_sequences.val then expr_wh else expr
-           in
-           pprintf pc "%p@ where rec %p = do {@;%p@ }" expr e (hlist patt) pl
-             (vlistl (semi_after (comm_expr expr_wh)) (comm_expr expr_wh)) el)
+      let expr_wh = if flag_where_in_sequences.val then expr_wh else expr in
+      pprintf pc "%p@ where rec %p = do {@;%p@ }" expr e (hlist patt) pl
+        (hvlistl (semi_after (comm_expr expr_wh)) (comm_expr expr_wh)) el
   | None ->
       pprintf pc "%p@ where rec %p =@;%p" expr e (hlist patt) pl
         (comm_expr expr) body ]
@@ -557,14 +550,6 @@ value has_cons_with_params vdl =
     vdl
 ;
 
-value rec get_else_if =
-  fun
-  [ <:expr< if $e1$ then $e2$ else $e3$ >> ->
-      let (eel, e3) = get_else_if e3 in
-      ([(e1, e2) :: eel], e3)
-  | e -> ([], e) ]
-;
-
 value alone_in_line pc =
   (pc.aft = "" || pc.aft = ";") && pc.bef <> "" &&
   loop 0 where rec loop i =
@@ -607,6 +592,90 @@ value are_close f x1 x2 =
   | _ -> False ]
 ;
 
+(* if statement *)
+
+value rec get_else_if =
+  fun
+  [ <:expr< if $e1$ then $e2$ else $e3$ >> ->
+      let (eel, e3) = get_else_if e3 in
+      ([(e1, e2) :: eel], e3)
+  | e -> ([], e) ]
+;
+
+value if_then force_vertic curr pc (e1, e2) =
+  let expr_wh = if flag_where_in_sequences.val then expr_wh else expr in
+  horiz_vertic_if force_vertic
+    (fun () -> pprintf pc "if %p then %p" curr e1 curr e2)
+    (fun () ->
+       let if_e1_then pc () = pprintf pc "@[<3>if %p@]@ then" curr e1 in
+       match sequencify e2 with
+       [ Some el ->
+           pprintf pc "%p do {@;%p@ }" if_e1_then () sequence el
+       | None ->
+           pprintf pc "%p@;%p" if_e1_then () (comm_expr expr_wh) e2 ])
+;
+
+value else_if_then force_vertic curr pc (e1, e2) =
+  let expr_wh = if flag_where_in_sequences.val then expr_wh else expr in
+  horiz_vertic_if force_vertic
+    (fun () ->
+       pprintf pc "else if %p then %p" curr e1 curr e2)
+    (fun () ->
+       let if_e1_then pc () = pprintf pc "@[<a>else if@;%p@ then@]" curr e1 in
+       match sequencify e2 with
+       [ Some el ->
+           pprintf pc "%p do {@;%p@ }" if_e1_then () sequence el
+       | None ->
+           pprintf pc "%p@;%p" if_e1_then () (comm_expr expr_wh) e2 ])
+;
+
+value rec loop_else_if force_vertic curr pc =
+  fun
+  [ [(e1, e2) :: eel] ->
+      pprintf pc "@[<b>@ %p%p@]" (else_if_then force_vertic curr) (e1, e2)
+        (loop_else_if force_vertic curr) eel
+  | [] ->
+      pprintf pc "" ]
+;
+
+value ending_else force_vertic curr pc e3 =
+  let expr_wh = if flag_where_in_sequences.val then expr_wh else expr in
+  horiz_vertic_if force_vertic
+    (fun () ->
+       pprintf pc "else %p" (comm_expr curr) e3)
+    (fun () ->
+       match sequencify e3 with
+       [ Some el -> pprintf pc "else do {@;%p@ }" sequence el
+       | None -> pprintf pc "else@;%p" (comm_expr expr_wh) e3 ])
+;
+
+value if_case_has_vertic curr pc e1 e2 eel e3 =
+  horiz_vertic
+    (fun () ->
+       let _ : string = if_then False curr {(pc) with aft = ""} (e1, e2) in
+       False)
+    (fun () -> True) ||
+  List.exists
+    (fun (e1, e2) ->
+       horiz_vertic
+         (fun () ->
+            let _ : string =
+              else_if_then False curr {(pc) with bef = tab pc.ind; aft = ""}
+                (e1, e2)
+            in
+            False)
+         (fun () -> True))
+    eel ||
+  horiz_vertic
+    (fun () ->
+       let _ : string =
+         let pc = {(pc) with bef = tab pc.ind} in
+         pprintf pc "else %p" (comm_expr curr) e3
+       in
+       False)
+    (fun () -> True)
+;
+
 (* Expressions displayed without spaces separating elements; special
    for expressions as strings or arrays indexes (x.[...] or x.(...)).
    Applied only if only containing +, -, *, /, integers and variables. *)
@@ -743,30 +812,10 @@ EXTEND_PRINTER
   pr_expr:
     [ "top"
       [ <:expr< if $e1$ then $e2$ else $e3$ >> ->
-          let expr_wh = if flag_where_after_then.val then expr_wh else expr in
           horiz_vertic
             (fun () ->
                pprintf pc "if %p then %p else %p" curr e1 curr e2 curr e3)
             (fun () ->
-               let if_then force_vertic pc else_b e1 e2 =
-                 horiz_vertic_if force_vertic
-                   (fun () ->
-                      pprintf pc "%sif %p then %p" else_b curr e1 curr e2)
-                   (fun () ->
-                      let if_e1_then pc () =
-                        if else_b = "" then
-                          pprintf pc "@[<3>if %p@]@ then" curr e1
-                        else
-                          pprintf pc "@[<a>%sif@;%p@ then@]" else_b curr e1
-                      in
-                      match sequencify e2 with
-                      [ Some el ->
-                          pprintf pc "%p do {@;%p@ }" if_e1_then ()
-                            sequence el
-                      | None ->
-                          pprintf pc "%p@;%p" if_e1_then ()
-                            (comm_expr expr_wh) e2 ])
-               in
                let (force_vertic, eel, e3) =
                  if flag_equilibrate_cases.val then
                    let (eel, e3) =
@@ -780,74 +829,16 @@ EXTEND_PRINTER
                      else get_else_if e3
                    in
                    (* if a case does not fit on line, all cases must be cut *)
-                   let has_vertic =
-                     horiz_vertic
-                       (fun () ->
-                          let _ : string =
-                            if_then False {(pc) with aft = ""} "" e1 e2
-                          in
-                          False)
-                       (fun () -> True) ||
-                     List.exists
-                       (fun (e1, e2) ->
-                          horiz_vertic
-                            (fun () ->
-                               let _ : string =
-                                 if_then False
-                                   {(pc) with bef = tab pc.ind; aft = ""}
-                                   "else " e1 e2
-                               in
-                               False)
-                            (fun () -> True))
-                       eel ||
-                     horiz_vertic
-                       (fun () ->
-                          let _ : string =
-                            sprintf "%selse %s%s" (tab pc.ind)
-                              (comm_expr curr {(pc) with bef = ""; aft = ""}
-                                 e3)
-                              pc.aft
-                          in
-                          False)
-                       (fun () -> True)
-                   in
+                   let has_vertic = if_case_has_vertic curr pc e1 e2 eel e3 in
                    (has_vertic, eel, e3)
                  else
                    let (eel, e3) = get_else_if e3 in
                    (False, eel, e3)
                in
-               let s1 = if_then force_vertic {(pc) with aft = ""} "" e1 e2 in
-               let s2 =
-                 loop eel where rec loop =
-                   fun
-                   [ [(e1, e2) :: eel] ->
-                       sprintf "\n%s%s"
-                         (if_then force_vertic
-                            {(pc) with bef = tab pc.ind; aft = ""}
-                            "else " e1 e2)
-                         (loop eel)
-                   | [] -> "" ]
-               in
-               let s3 =
-                 horiz_vertic_if force_vertic
-                   (fun () ->
-                      let pc = {(pc) with bef = tab pc.ind} in
-                      pprintf pc "else %p" (comm_expr curr) e3)
-                   (fun () ->
-                      match sequencify e3 with
-                      [ Some el ->
-                          sequence_box
-                            (fun pc () ->
-                               horiz_vertic (fun () -> sprintf "\n")
-                                 (fun () ->
-                                    let pc = {(pc) with bef = tab pc.ind} in
-                                    pprintf pc "else"))
-                            pc el
-                      | None ->
-                          let pc = {(pc) with bef = tab pc.ind} in
-                          pprintf pc "else@;%p" (comm_expr expr_wh) e3 ])
-               in
-               sprintf "%s%s\n%s" s1 s2 s3)
+               pprintf pc "@[<b>%p%p@ %p@]"
+                 (if_then force_vertic curr) (e1, e2)
+                 (loop_else_if force_vertic curr) eel
+                 (ending_else force_vertic curr) e3)
       | <:expr< fun [ $list:pwel$ ] >> ->
           match pwel with
           [ [(p1, <:vala< None >>, e1)] when is_irrefut_patt p1 ->
@@ -934,8 +925,14 @@ EXTEND_PRINTER
           let expr_wh = if flag_where_after_in.val then expr_wh else curr in
           horiz_vertic_if (not flag_horiz_let_in.val)
             (fun () ->
-               pprintf pc "let %s%p in %p" (if rf then "rec " else "")
-                 (hlist2 let_binding (and_before let_binding)) pel curr e)
+               match flatten_sequence ge with
+               [ Some el ->
+                   let loc = MLast.loc_of_expr ge in
+                   curr pc <:expr< do { $list:el$ } >>
+               | None ->
+                   pprintf pc "let %s%p in %p" (if rf then "rec " else "")
+                     (hlist2 let_binding (and_before let_binding)) pel
+                     curr e ])
             (fun () ->
                match flatten_sequence ge with
                [ Some el ->
@@ -1144,6 +1141,7 @@ EXTEND_PRINTER
         <:expr< for $lid:_$ = $_$ $to:_$ $_$ do { $list:_$ } >> |
         <:expr< while $_$ do { $list:_$ } >> |
         <:expr< let $flag:_$ $list:_$ in $_$ >> |
+        <:expr< let module $uid:_$ = $_$ in $_$ >> |
         <:expr< match $_$ with [ $list:_$ ] >> |
         <:expr< try $_$ with [ $list:_$ ] >> as z ->
           let expr_wh =
@@ -1271,13 +1269,15 @@ EXTEND_PRINTER
                pprintf pc "@[<2>{ %p }@]"
                  (vlistl (semi_after label_decl) label_decl) ltl)
       | <:ctyp< [ $list:vdl$ ] >> ->
-          horiz_vertic_if (has_cons_with_params vdl)
-            (fun () ->
-               pprintf pc "[ %p ]" (hlist2 cons_decl (bar_before cons_decl))
-                 vdl)
-            (fun () ->
-               pprintf pc "[ %p ]" (vlist2 cons_decl (bar_before cons_decl))
-                 vdl)
+          if vdl = [] then pprintf pc "[]"
+          else
+            horiz_vertic_if (has_cons_with_params vdl)
+              (fun () ->
+                 pprintf pc "[ %p ]" (hlist2 cons_decl (bar_before cons_decl))
+                   vdl)
+              (fun () ->
+                 pprintf pc "[ %p ]" (vlist2 cons_decl (bar_before cons_decl))
+                   vdl)
       | <:ctyp< ($list:tl$) >> ->
           let tl = List.map (fun t -> (t, " *")) tl in
           pprintf pc "@[<1>(%p)@]" (plist ctyp 0) tl
@@ -1409,7 +1409,9 @@ EXTEND_PRINTER
           pprintf pc "%s" s
       | <:module_expr< ($me$ : $mt$) >> ->
           pprintf pc "@[<1>(%p :@ %p)@]" module_expr me module_type mt
-      | <:module_expr< struct $list:_$ end >> as z ->
+      | <:module_expr< functor ($uid:_$ : $_$) -> $_$ >> |
+        <:module_expr< struct $list:_$ end >> | <:module_expr< $_$ . $_$ >> |
+        <:module_expr< $_$ $_$ >> as z ->
           pprintf pc "@[<1>(%p)@]" module_expr z ] ]
   ;
   pr_module_type:
