@@ -1,6 +1,6 @@
-(* camlp5r pa_macro.cmo q_MLast.cmo ./pa_extprint.cmo ./pa_extfun.cmo *)
-(* $Id: pr_scheme.ml 1517 2007-11-14 10:15:20Z deraugla $ *)
-(* Copyright (c) INRIA 2007 *)
+(* camlp5r pa_macro.cmo q_MLast.cmo ./pa_extprint.cmo ./pa_extfun.cmo ./pa_pprintf.cmo *)
+(* $Id: pr_scheme.ml 1832 2007-12-29 04:26:28Z deraugla $ *)
+(* Copyright (c) INRIA 2007-2008 *)
 
 open Pretty;
 open Pcaml;
@@ -22,16 +22,20 @@ do {
 
 (* general functions *)
 
+(**)
+value test = ref False;
+Pcaml.add_option "-test" (Arg.Set test) " test";
+(**)
+
 value not_impl name pc x =
   let desc =
     if Obj.tag (Obj.repr x) = Obj.tag (Obj.repr "") then
-      sprintf "\"%s\"" (Obj.magic x)
+      "\"" ^ Obj.magic x ^ "\""
     else if Obj.is_block (Obj.repr x) then
       "tag = " ^ string_of_int (Obj.tag (Obj.repr x))
     else "int_val = " ^ string_of_int (Obj.magic x)
   in
-  sprintf "%s\"pr_scheme, not impl: %s; %s\"%s" pc.bef name
-    (String.escaped desc) pc.aft
+  pprintf pc "\"pr_scheme, not impl: %s; %s\"" name (String.escaped desc)
 ;
 
 value to_be_renamed = ["cond"; "sum"];
@@ -40,9 +44,9 @@ value rename_id s = if List.mem s to_be_renamed then s ^ "#" else s;
 
 value rec longident pc sl =
   match sl with
-  [ [] -> sprintf "%s%s" pc.bef pc.aft
-  | [s] -> sprintf "%s%s%s" pc.bef s pc.aft
-  | [s :: sl] -> longident {(pc) with bef = sprintf "%s%s." pc.bef s} sl ]
+  [ [] -> pprintf pc ""
+  | [s] -> pprintf pc "%s" s
+  | [s :: sl] -> pprintf pc "%s.%p" s longident sl ]
 ;
 
 (*
@@ -114,38 +118,26 @@ value brace pc b =
    aft = sprintf "}%s" pc.aft}
 ;
 
-value type_param pc (s, (pl, mi)) =
-  sprintf "%s%s'%s%s" pc.bef
-    (if pl then "+" else if mi then "-" else "") (Pcaml.unvala s) pc.aft
+value braceless pc b =
+  {(pc) with ind = pc.ind + 1; bef = sprintf "%s{<%s" pc.bef b;
+   aft = sprintf ">}%s" pc.aft}
 ;
 
+value type_param pc (s, (pl, mi)) =
+  pprintf pc "%s'%s" (if pl then "+" else if mi then "-" else "")
+    (Pcaml.unvala s)
+;
+
+value type_var pc v = pprintf pc "'%s" v;
+
 value type_decl b pc td =
   let n = rename_id (Pcaml.unvala (snd td.MLast.tdNam)) in
-  horiz_vertic
-    (fun () ->
-       sprintf "%s(%s%s %s)%s" pc.bef b
-         (match Pcaml.unvala td.MLast.tdPrm with
-          [ [] -> n
-          | tp ->
-              sprintf "(%s %s)" n
-                (hlist type_param {(pc) with bef = ""; aft = ""} tp) ])
-         (ctyp {(pc) with bef = ""; aft = ""} td.MLast.tdDef) pc.aft)
-    (fun () ->
-       let s1 =
-         sprintf "%s(%s%s" pc.bef b
-           (match Pcaml.unvala td.MLast.tdPrm with
-            [ [] -> n
-            | tp ->
-                sprintf "(%s %s)" n
-                  (hlist type_param {(pc) with bef = ""; aft = ""} tp) ])
-       in
-       let s2 =
-         ctyp
-           {(pc) with ind = pc.ind + 1; bef = tab (pc.ind + 1);
-            aft = sprintf ")%s" pc.aft}
-           td.MLast.tdDef
-       in
-       sprintf "%s\n%s" s1 s2)
+  pprintf pc "@[<1>(%s%p@ %p)@]" b
+    (fun pc ->
+       fun
+       [ [] -> pprintf pc "%s" n
+       | tp -> pprintf pc "(%s %p)" n (hlist type_param) tp ])
+    (Pcaml.unvala td.MLast.tdPrm) ctyp td.MLast.tdDef
 ;
 
 value type_decl_list pc =
@@ -153,24 +145,13 @@ value type_decl_list pc =
   [ [td] -> type_decl "type " pc td
   | tdl ->
       horiz_vertic
-        (fun () ->
-           sprintf "%s(type* %s)%s" pc.bef
-             (hlist (type_decl "") {(pc) with bef = ""; aft = ""} tdl)
-             pc.aft)
-        (fun () ->
-           let s1 = sprintf "%s(type*" pc.bef in
-           let s2 =
-             vlist (type_decl "")
-               {(pc) with ind = pc.ind + 1; bef = tab (pc.ind + 1);
-                aft = sprintf ")%s" pc.aft}
-               tdl
-           in
-           sprintf "%s\n%s" s1 s2) ]
+        (fun () -> pprintf pc "(type* %p)" (hlist (type_decl "")) tdl)
+        (fun () -> pprintf pc "(type*@;<1 1>%p)" (vlist (type_decl "")) tdl) ]
 ;
 
 value exception_decl pc (c, tl) =
   plistbf 0 (paren pc "exception")
-    [(fun pc -> sprintf "%s%s%s" pc.bef c pc.aft, "") ::
+    [(fun pc -> pprintf pc "%s" c, "") ::
      List.map (fun t -> (fun pc -> ctyp pc t, "")) tl]
 ;
 
@@ -178,23 +159,18 @@ value value_binding b pc (p, e) =
   let (pl, e) = expr_fun_args e in
   horiz_vertic
     (fun () ->
-       let s =
-         match pl with
-         [ [] -> patt {(pc) with bef = ""; aft = ""} p
-         | _ -> hlist patt {(pc) with bef = "("; aft = ")"} [p :: pl] ]
-       in
-       sprintf "%s(%s%s %s)%s" pc.bef (if b = "" then "" else b ^ " ") s
-         (expr {(pc) with bef = ""; aft = ""} e) pc.aft)
+       pprintf pc "(%s%p %p)" (if b = "" then "" else b ^ " ")
+         (fun pc ->
+            fun
+            [ [] -> patt pc p
+            | _ -> pprintf pc "(%p)" (hlist patt) [p :: pl] ])
+         pl expr e)
     (fun () ->
        let s1 =
          match pl with
          [ [] ->
-             let pc =
-               {(pc) with ind = pc.ind + 1;
-                bef = sprintf "%s(%s" pc.bef (if b = "" then "" else b ^ " ");
-                aft = ""}
-             in
-             patt pc p
+             let pc = {(pc) with aft = ""} in
+             pprintf pc "@[<1>(%s%p@]" (if b = "" then "" else b ^ " ") patt p
          | _ ->
              let pc =
                {(pc) with ind = pc.ind + 1; bef = sprintf "%s(%s" pc.bef b;
@@ -222,18 +198,9 @@ value value_binding_list pc (rf, pel) =
   | _ ->
       horiz_vertic
         (fun () ->
-           sprintf "%s(%s* %s)%s" pc.bef b
-             (hlist (value_binding "") {(pc) with bef = ""; aft = ""} pel)
-             pc.aft)
+           pprintf pc "(%s* %p)" b (hlist (value_binding "")) pel)
         (fun () ->
-           let s1 = sprintf "%s(%s*" pc.bef b in
-           let s2 =
-             vlist (value_binding "")
-               {(pc) with ind = pc.ind + 1; bef = tab (pc.ind + 1);
-                aft = sprintf ")%s" pc.aft}
-               pel
-           in
-           sprintf "%s\n%s" s1 s2) ]
+           pprintf pc "(%s*@;<1 1>%p)" b (vlist (value_binding "")) pel) ]
 ;
 
 value let_binding pc (p, e) =
@@ -242,11 +209,7 @@ value let_binding pc (p, e) =
     [(fun pc ->
         match pl with
         [ [] -> patt pc p
-        | _ ->
-            hlist patt
-              {(pc) with bef = sprintf "%s(" pc.bef;
-               aft = sprintf ")%s" pc.aft}
-              [p :: pl] ],
+        | _ -> pprintf pc "(%p)" (hlist patt) [p :: pl] ],
       "");
      (fun pc -> expr pc e, "")]
 ;
@@ -280,27 +243,26 @@ value constr_decl pc (_, c, tl) =
   let c = Pcaml.unvala c in
   let tl = Pcaml.unvala tl in
   match tl with
-  [ [] -> sprintf "%s(%s)%s" pc.bef c pc.aft
+  [ [] -> pprintf pc "(%s)" c
   | _ ->
       plistf 0 (paren pc "")
-        [(fun pc -> sprintf "%s%s%s" pc.bef c pc.aft, "") ::
+        [(fun pc -> pprintf pc "%s" c, "") ::
          List.map (fun t -> (fun pc -> ctyp pc t, "")) tl] ]
 ;
 
 value poly_variant_decl pc =
   fun
   [ <:poly_variant< `$s$ >> ->
-      sprintf "%s(` %s)%s" pc.bef s pc.aft
+      pprintf pc "(` %s)" s
   | <:poly_variant< `$s$ of $flag:a$ $list:tl$ >> ->
       let list =
         let list =
           [(fun pc -> plist ctyp 0 pc (List.map (fun t -> (t, "")) tl), "")]
         in
         let list =
-          if a then [(fun pc -> sprintf "%s&%s" pc.bef pc.aft, "") :: list]
-          else list
+          if a then [(fun pc -> pprintf pc "&", "") :: list] else list
         in
-        [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "") :: list]
+        [(fun pc -> pprintf pc "%s" s, "") :: list]
       in
       plistbf 0 (paren pc "`") list
   | <:poly_variant< $t$ >> ->
@@ -313,18 +275,22 @@ value poly_variant_decl pc =
 value label_decl pc (_, l, m, t) =
   let list = [(fun pc -> ctyp pc t, "")] in
   plistf 0 (paren pc "")
-    [(fun pc -> sprintf "%s%s%s" pc.bef l pc.aft, "") ::
-     if m then [(fun pc -> sprintf "%smutable%s" pc.bef pc.aft, "") :: list]
-     else list]
+    [(fun pc -> pprintf pc "%s" l, "") ::
+     if m then [(fun pc -> pprintf pc "mutable", "") :: list] else list]
 ;
 
 value module_type_decl pc (s, mt) =
   plistbf 0 (paren pc "moduletype")
-    [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "");
+    [(fun pc -> pprintf pc "%s" s, "");
      (fun pc -> module_type pc mt, "")]
 ;
 
-value string pc s = sprintf "%s\"%s\"%s" pc.bef s pc.aft;
+value field_expr pc (l, e) =
+  plistf 0 (paren pc "")
+    [(fun pc -> pprintf pc "%s" l, ""); (fun pc -> expr pc e, "")]
+;
+
+value string pc s = pprintf pc "\"%s\"" s;
 
 value int_repr s =
   if String.length s > 2 && s.[0] = '0' then
@@ -345,17 +311,14 @@ value int_repr s =
 value with_constraint pc =
   fun
   [ <:with_constr< type $sl$ $list:tvl$ = $flag:pf$ $t$ >> ->
-      horiz_vertic
-        (fun () ->
-           let n = longident {(pc) with bef = ""; aft = ""} sl in
-           sprintf "%s(type%s %s %s)%s" pc.bef (if pf then "private" else "")
-             (match tvl with
-              [ [] -> n
-              | tvl ->
-                  sprintf "(%s %s)" n
-                    (hlist type_param {(pc) with bef = ""; aft = ""} tvl) ])
-             (ctyp {(pc) with bef = ""; aft = ""} t) pc.aft)
-        (fun () -> not_impl "with_constraint type vertic" pc 0)
+      pprintf pc "(type%s %p@;<1 1>%p)" (if pf then "private" else "")
+        (fun pc ->
+           fun
+           [ [] -> longident pc sl
+           | tvl ->
+               pprintf pc "(%p %p)" longident sl
+                 (hlist type_param) tvl ])
+        tvl ctyp t
   | wc -> not_impl "with_constraint" pc wc ]
 ;
 
@@ -363,20 +326,19 @@ value class_descr b pc cd =
   let n = Pcaml.unvala cd.MLast.ciNam in
   horiz_vertic
     (fun () ->
-       sprintf "%s(%s%s%s %s)%s" pc.bef (if b = "" then "" else b ^ " ")
+       pprintf pc "(%s%s%p %p)" (if b = "" then "" else b ^ " ")
          (if Pcaml.unvala cd.MLast.ciVir then "virtual " else "")
-         (match Pcaml.unvala (snd cd.MLast.ciPrm) with
-          [ [] -> n
-          | tvl ->
-              sprintf "(%s %s)" n
-                (hlist type_param {(pc) with bef = ""; aft = ""} tvl) ])
-         (class_type {(pc) with bef = ""; aft = ""} cd.MLast.ciExp) pc.aft)
+         (fun pc ->
+            fun
+            [ [] -> pprintf pc "%s" n
+            | tvl -> pprintf pc "(%s %p)" n (hlist type_param) tvl ])
+         (Pcaml.unvala (snd cd.MLast.ciPrm)) class_type cd.MLast.ciExp)
     (fun () ->
        let list =
          let list =
            [(fun pc ->
                match Pcaml.unvala (snd cd.MLast.ciPrm) with
-               [ [] -> sprintf "%s%s%s" pc.bef n pc.aft
+               [ [] -> pprintf pc "%s" n
                | tvl ->
                    plistb type_param 0 (paren pc n)
                      (List.map (fun tv -> (tv, "")) tvl) ],
@@ -385,12 +347,11 @@ value class_descr b pc cd =
          in
          let list =
            if Pcaml.unvala cd.MLast.ciVir then
-             [(fun pc -> sprintf "%svirtual%s" pc.bef pc.aft, "") ::
-              list]
+             [(fun pc -> pprintf pc "virtual", "") :: list]
            else list
          in
          if b = "" then list
-         else [(fun pc -> sprintf "%s%s%s" pc.bef b pc.aft, "") :: list]
+         else [(fun pc -> pprintf pc "%s" b, "") :: list]
        in
        plistf 0 (paren pc "") list)
 ;
@@ -400,10 +361,7 @@ value class_descr_list pc =
   [ [cd] -> class_descr "class" pc cd
   | cdl ->
       horiz_vertic
-        (fun () ->
-           sprintf "%s(class* %s)%s" pc.bef
-             (hlist (class_descr "") {(pc) with bef = ""; aft = ""} cdl)
-             pc.aft)
+        (fun () -> pprintf pc "(class* %p)" (hlist (class_descr "")) cdl)
         (fun () ->
            let s1 = sprintf "%s(class*" pc.bef in
            let s2 =
@@ -452,6 +410,43 @@ value class_decl b pc cd =
        plistf 0 (paren pc "") list)
 ;
 
+value class_type_decl b pc cd =
+  let n = Pcaml.unvala cd.MLast.ciNam in
+  let ct = cd.MLast.ciExp in
+  horiz_vertic
+    (fun () ->
+       sprintf "%s(%s%s%s %s)%s" pc.bef (if b = "" then "" else b ^ " ")
+         (if Pcaml.unvala cd.MLast.ciVir then "virtual " else "")
+         (match Pcaml.unvala (snd cd.MLast.ciPrm) with
+          [ [] -> n
+          | tvl ->
+              sprintf "(%s %s)" n
+                (hlist type_param {(pc) with bef = ""; aft = ""} tvl) ])
+         (class_type {(pc) with bef = ""; aft = ""} ct) pc.aft)
+    (fun () ->
+       let list =
+         let list =
+           [(fun pc ->
+               match Pcaml.unvala (snd cd.MLast.ciPrm) with
+               [ [] -> sprintf "%s%s%s" pc.bef n pc.aft
+               | tvl ->
+                   plistb type_param 0 (paren pc n)
+                     (List.map (fun tv -> (tv, "")) tvl) ],
+             "");
+            (fun pc -> class_type pc ct, "")]
+         in
+         let list =
+           if Pcaml.unvala cd.MLast.ciVir then
+             [(fun pc -> sprintf "%svirtual%s" pc.bef pc.aft, "") ::
+              list]
+           else list
+         in
+         if b = "" then list
+         else [(fun pc -> sprintf "%s%s%s" pc.bef b pc.aft, "") :: list]
+       in
+       plistf 0 (paren pc "") list)
+;
+
 value class_decl_list pc =
   fun
   [ [cd] -> class_decl "class" pc cd
@@ -472,6 +467,26 @@ value class_decl_list pc =
            sprintf "%s\n%s" s1 s2) ]
 ;
 
+value class_type_decl_list pc =
+  fun
+  [ [ctd] -> class_type_decl "class" pc ctd
+  | ctdl ->
+      horiz_vertic
+        (fun () ->
+           sprintf "%s(class* %s)%s" pc.bef
+             (hlist (class_type_decl "") {(pc) with bef = ""; aft = ""} ctdl)
+             pc.aft)
+        (fun () ->
+           let s1 = sprintf "%s(classtype*" pc.bef in
+           let s2 =
+             vlist (class_type_decl "")
+               {(pc) with ind = pc.ind + 1; bef = tab (pc.ind + 1);
+                aft = sprintf ")%s" pc.aft}
+               ctdl
+           in
+           sprintf "%s\n%s" s1 s2) ]
+;
+
 EXTEND_PRINTER
   pr_ctyp:
     [ "top"
@@ -554,6 +569,10 @@ EXTEND_PRINTER
           sprintf "%s'%s%s" pc.bef s pc.aft
       | <:ctyp< _ >> ->
           sprintf "%s_%s" pc.bef pc.aft
+      | <:ctyp< # $list:sl$ >> ->
+          pprintf pc "(# %p)" longident sl
+      | <:ctyp< ! $list:pl$ . $t$ >> ->
+          pprintf pc "(! (%p)@;<1 1>%p)" (hlist type_var) pl ctyp t
       | x ->
           not_impl "ctyp" pc x ] ]
   ;
@@ -856,6 +875,35 @@ EXTEND_PRINTER
           sprintf "%s'%s'%s" pc.bef s pc.aft
       | <:expr< ` $s$ >> ->
           sprintf "%s(` %s)%s" pc.bef s pc.aft
+      | <:expr< {< $list:fel$ >} >> ->
+          let fel = List.map (fun fel -> (fel, "")) fel in
+          plist field_expr 0 (braceless pc "") fel
+      | <:expr< object $opt:cst$ $list:csl$ end >> ->
+          horiz_vertic
+            (fun () ->
+               sprintf "%s(object%s %s)%s" pc.bef
+                 (match cst with
+                  [ Some t -> not_impl "expr object self horiz " pc 0
+                  | None -> "" ])
+                 (hlist class_str_item {(pc) with bef = ""; aft = ""} csl)
+                    pc.aft)
+            (fun () ->
+               let s1 =
+                 let s = sprintf "%s(object" pc.bef in
+                 match cst with
+                 [ Some p ->
+                     plistb patt 0
+                       {(pc) with ind = pc.ind + 1; bef = s; aft = ""}
+                       [(p, "")]
+                 | None -> s ]
+               in
+               let s2 =
+                 vlist class_str_item
+                   {(pc) with ind = pc.ind + 1; bef = tab (pc.ind + 1);
+                    aft = sprintf ")%s" pc.aft}
+                   csl
+               in
+               sprintf "%s\n%s" s1 s2)
       | x ->
           not_impl "expr" pc x ] ]
   ;
@@ -984,6 +1032,8 @@ EXTEND_PRINTER
           sprintf "%s(` %s)%s" pc.bef s pc.aft
       | <:patt< _ >> ->
           sprintf "%s_%s" pc.bef pc.aft
+      | <:patt< # $list:sl$ >> ->
+          pprintf pc "(# %p)" longident sl
       | x ->
           not_impl "patt" pc x ] ]
   ;
@@ -991,6 +1041,8 @@ EXTEND_PRINTER
     [ "top"
       [ <:str_item< class $list:cdl$ >> ->
           class_decl_list pc cdl
+      | <:str_item< class type $list:ctdl$ >> ->
+          class_type_decl_list pc ctdl
       | <:str_item< open $i$ >> ->
           plistb longident 0 (paren pc "open") [(i, "")]
       | <:str_item< include $me$ >> ->
@@ -1046,6 +1098,8 @@ EXTEND_PRINTER
             [(fun pc -> sprintf "%s%s%s" pc.bef i pc.aft, "");
              (fun pc -> ctyp pc t, "") ::
              List.map (fun s -> (fun pc -> string pc s, "")) pd]
+      | <:sig_item< include $mt$ >> ->
+          plistb module_type 0 (paren pc "include") [(mt, "")]
       | <:sig_item< module $uid:s$ : $mt$ >> ->
           plistbf 0 (paren pc "module")
             [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "");
@@ -1126,13 +1180,7 @@ EXTEND_PRINTER
                in
                sprintf "%s\n%s" s1 s2)
       | <:module_type< $mt$ with $list:wcl$ >> ->
-          horiz_vertic
-            (fun () ->
-               sprintf "%s(with %s %s)%s" pc.bef
-                 (curr {(pc) with bef = ""; aft = ""} mt)
-                 (hlist with_constraint {(pc) with bef = ""; aft = ""} wcl)
-                 pc.aft)
-            (fun () -> not_impl "module_type with vertic" pc 0)
+          pprintf pc "(with %p@;<1 1>%p)" curr mt (hlist with_constraint) wcl
       | <:module_type< $me1$ . $me2$ >> ->
            sprintf "%s.%s"
              (curr {(pc) with aft = ""} me1)
@@ -1219,6 +1267,8 @@ EXTEND_PRINTER
             else list
           in
           plistbf 0 (paren pc "value") list
+      | <:class_str_item< type $t1$ = $t2$ >> ->
+          pprintf pc "(type %p@;<1 1>%p)" ctyp t1 ctyp t2
       | x ->
           not_impl "class_str_item" pc x ] ]
   ;
