@@ -1,6 +1,6 @@
 (* camlp5r *)
-(* $Id: fstream.mli 1563 2007-11-26 09:20:52Z deraugla $ *)
-(* Copyright (c) INRIA 2007 *)
+(* $Id: fstream.mli 1804 2007-12-27 10:30:24Z deraugla $ *)
+(* Copyright (c) INRIA 2007-2008 *)
 
 (* Module [Fstream]: functional streams *)
 
@@ -90,4 +90,4 @@ value flazy : (unit -> data 'a) -> t 'a;
 value b_seq : bp 'a 'b -> ('b -> bp 'a 'c) -> bp 'a 'c;
 value b_or : bp 'a 'b -> bp 'a 'b -> bp 'a 'b;
 value b_term : ('a -> option 'b) -> bp 'a 'b;
-value b_nok : kont _ _;
+value b_act : 'b -> bp 'a 'b;
