@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: apply.sh 673 2007-07-11 12:01:39Z deraugla $
+# $Id: apply.sh 1689 2007-12-16 21:46:19Z deraugla $
 
 ARGS1=
 ARGS2=
@@ -29,5 +29,5 @@ else
 fi
 
 OTOP=../..
-echo ocamlrun $COMM $ARGS1 $ARGS2 $ARGS3 $FILE 1>&2
+# echo ocamlrun $COMM $ARGS1 $ARGS2 $ARGS3 $FILE 1>&2
 ocamlrun $COMM $ARGS1 $ARGS2 $ARGS3 $FILE
